package Export;

import Topology.*;
import Graph.*;
import Model.*;
import Util.*;

import java.io.*;
import java.util.*;

//Konstantinos Katsaros. Edited by Giorgos Nomikos to export topology for Omnet++ version 4.
/**
 * Export filter for OMNeT++
 */
public class Omnetpp4Export
{

    // Konstantinos Katsaros, Giorgos Nomikos
    //---------------------------------------------------------------------------
    // Please don't forget to change the units below if you don't use sec and bps
    // Uncomment **Delay vars below and replace rec.getLength() later for fixed
    // and predetermined propagation delays.
    //  SET YOUR PREFERED VALUES HERE
    // Transit-transit link
    //double ttDelay = 0.100; //0.0018; //in sec
    double ttError = 1e-15;
    double ttDatarate = 104857600; //128000; //bps
    // Transit-stub link
    //double tsDelay = 0.020; //0.0017;
    double tsError = 1e-16;
    double tsDatarate = 104857600; //17000;
    // Stub-stub link
    //double ssDelay = 0.005; //0.0016;
    double ssError = 1e-17;
    double ssDatarate = 104857600; //126000;
    //---------------------------------------------------------------------------
    
    private Topology t;
    private BufferedWriter bw;
    private BufferedWriter bw2;
    //private BufferedWriter bw3;
    private BufferedWriter bw4;
    private BufferedReader br;
    private String channelsFullFileName;
    private String channelsFileName;

    // Konstantinos Katsaros: 21/05/2008
    // HashMaps that will help as keep the mapping between BRITE node IDs and their index in the created
    // router sub-module arrays. The id2indexOverlay will help as keep track of overlay participation.
    private HashMap id2indexAccess;
    private HashMap id2indexBackbone;
    private HashMap id2indexOverlay; //id2indexOverlay.second = 1 => in overlay
    private HashMap id2indexAccessArray; //.second = 1 => is an access router

    public Omnetpp4Export(Topology t, File outFile)
    {
        this.t = t;
        try
        {
            bw = new BufferedWriter(new FileWriter(outFile + ".ned") );

            // Konstantinos Katsaros: 25/06/2008
            bw2 = new BufferedWriter(new FileWriter(new File(outFile + "-ASIDs.ini")));
            bw2.write("[General]\n");

            // Konstantinos Katsaros: 03/04/2009
            // Will write the number of access routers per stub domain.
            //bw3 = new BufferedWriter(new FileWriter(new File(outFile + "-AS_AccessRouterNum.ini")));
            //bw3.write("[General]\n");

            // Konstantinos Katsaros: 03/04/2010
            // Writing down InetUnderlay configuration parameters, The created file must be inluded
	    // in the scenario .ini file
            bw4 = new BufferedWriter(new FileWriter(new File(outFile + "-Config.ini")));
            bw4.write("[General]\n");

        }
        catch (IOException e)
        {
            Util.ERR("Error creating BufferedWriter in Omnetpp4Export: " + e);
        }
    }

    public void writeASIDinfo(int accessRouterIndex, int ASID)
    {
        try
        {
            bw2.write("**.accessRouter[" + accessRouterIndex + "].asid = " + ASID + "\n");
        }
        catch (Exception e)
        {
            Util.ERR("Error writting ASID parameter values: " + e);
        }
    }

/*
    public void writeASAccessRouterNumInfo(HashMap map)
    {
        try
        {
            bw.write("    ASInfoArray[" + map.size() + "]: ASInfo {\n");
	    bw.write("      parameters: \n");
            bw.write("      	@display(\"i=block/control\");\n");
	    bw.write("      }\n");

            int i = 0;
            for (Iterator it = map.entrySet().iterator(); it.hasNext();)
            {
                Map.Entry entry = (Map.Entry) it.next();
                Integer ASID = (Integer) entry.getKey();
                Integer RouterNumber = (Integer) entry.getValue();
                bw3.write("**.ASInfoArray[" + i + "].asid = " + ASID + "\n");
                bw3.write("**.ASInfoArray[" + i + "].ASaccessRouterNum = " + RouterNumber + "\n");
                i++;
            }
        }
        catch (Exception e)
        {
            Util.ERR("Error writting ASID parameter values: " + e);
        }
    }
*/
    public void export() throws Exception
    {
        Util.MSG("Producing export file for OMNeT++ version 4");
        Graph g = t.getGraph();
        FileModel model = (FileModel) t.getModel();
        double overlayParticipation = model.getOverlayParticipation();
        System.out.println("[EXPORT]: OverlayParticipation: " + overlayParticipation);

        // Konstantinos Katsaros: 21/05/2008
        id2indexAccess = new HashMap();
        id2indexBackbone = new HashMap();
        id2indexOverlay = new HashMap();
        id2indexAccessArray = new HashMap();
        HashMap ASRouterNumbers = new HashMap();

        Node[] nodes = g.getNodesArray();

        Arrays.sort(nodes, Node.IDcomparator);
        Edge[] edges = g.getEdgesArray();
        Arrays.sort(edges, Edge.SrcIDComparator);

        int maxid = 0;
        for (int i = 0; i < nodes.length; i++)
        {
            int id = nodes[i].getID();
            if (id > maxid)
            {
                maxid = id;
            }
        }
        int in_lastused[] = new int[maxid + 1];
        int out_lastused[] = new int[maxid + 1];
        for (int i = 0; i < maxid; i++)
        {
            in_lastused[i] = 0;
            out_lastused[i] = 0;
        }

        //Giorgos Nomikos 01/05/2009
        bw.write("//\n");
        bw.write("// OMNeT++ network description -- exported from GT-ITM (alt) topology\n");
        bw.write("// Generator Model Used:\n");
        StringTokenizer st = new StringTokenizer(t.getModel().toString(), "\n");
        while (st.hasMoreTokens())
        {
            bw.write("//   " + st.nextToken() + "\n");
        }
        bw.write("//\n");
        bw.write("\n");
        bw.write("//This file must be placed under oversim/src/underlay/inetunderlay\n\n");


        bw.write("//Package\n");
        bw.write("package oversim.underlay.inetunderlay;\n\n");
        bw.write("//\n");
        bw.write("// Prototype for modules that can be used as node in a GTITM_TransitStub_Network\n");
        bw.write("//\n\n");


        bw.write("//Imports\n");
        //bw.write("import oversim.common.ITier;\n");
        bw.write("import oversim.common.TierDummy;\n");
        bw.write("import oversim.common.IOverlay;\n");
        bw.write("import oversim.underlay.inetunderlay.Router;\n");
        bw.write("import oversim.underlay.inetunderlay.AccessRouter;\n");
        bw.write("import oversim.underlay.inetunderlay.OverlayRouter;\n");
        bw.write("import oversim.underlay.inetunderlay.OverlayAccessRouter;\n");
        bw.write("import oversim.underlay.inetunderlay.TunOutRouter;\n");
        bw.write("import oversim.underlay.inetunderlay.InetUnderlayConfigurator;\n");
        bw.write("import inet.nodes.inet.StandardHost;\n");
        bw.write("import oversim.common.GlobalObserver;\n");


        // Konstantinos Katsaros: 15/05/2008
        // We do this loop in order tou count the access routers. We need this info as it is retrieved by the IPv4UnderlayConfiguratorModule
        // Konstantinos Katsaros: 21/05/2008
        // In this first parsing we also keep the mapping between the BRITE node IDs and the corresponding sub-module arrays
        // (that will be produced later) indexes.
        int accessRouterNum = 0;
        int backboneRouterNum = 0;
        int accessRouterIndex = -1;
        int backboneRouterIndex = -1;
        int randomAccessOverlay = 0;
        int randomBackboneOverlay = 0;
        for (int i = 0; i < nodes.length; i++)
        {
            Node n = nodes[i];
            NodeConf nc = nodes[i].getNodeConf();
            RouterNodeConf rnc = (RouterNodeConf) nc;

            if (rnc.getType() == ModelConstants.RT_STUB)
            {
                id2indexAccessArray.put(new Integer(n.getID()), 1);
                accessRouterNum++;
                if (Math.random() < (overlayParticipation))
                {
                    randomAccessOverlay++;
                    //Storing in which array this node will be
                    id2indexOverlay.put(new Integer(n.getID()), 1);
                    //Storing its index there
                    id2indexAccess.put(new Integer(n.getID()), new Integer(randomAccessOverlay - 1));
                }
                else
                {
                    id2indexOverlay.put(new Integer(n.getID()), 0);
                    accessRouterIndex++;
                    id2indexAccess.put(new Integer(n.getID()), new Integer(accessRouterIndex));

                    writeASIDinfo(accessRouterIndex, rnc.getASID());
                    Integer ASIDobj = rnc.getASID();
                    Integer storedEntry = (Integer) ASRouterNumbers.get(ASIDobj);
                    if (storedEntry == null)
                    {
                        ASRouterNumbers.put(new Integer(rnc.getASID()), new Integer(1));
                    }
                    else
                    {
                        ASRouterNumbers.remove(ASIDobj);
                        ASRouterNumbers.put(ASIDobj, new Integer(storedEntry + 1));
                    }
                }



            }
            else if (rnc.getType() == ModelConstants.RT_BACKBONE)
            {
                id2indexAccessArray.put(new Integer(n.getID()), 0);
                backboneRouterNum++;
                if (Math.random() < (overlayParticipation))
                {
                    randomBackboneOverlay++;
                    id2indexOverlay.put(new Integer(n.getID()), 1);
                    id2indexBackbone.put(new Integer(n.getID()), new Integer(randomBackboneOverlay - 1));
                }
                else
                {
                    id2indexOverlay.put(new Integer(n.getID()), 0);
                    backboneRouterIndex++;
                    id2indexBackbone.put(new Integer(n.getID()), new Integer(backboneRouterIndex));
                }


            }
        }



        System.out.println("We have totaly:\n");
        System.out.println("AccessRouters = " + accessRouterNum);
        System.out.println("BackboneRouters = " + backboneRouterNum);
        System.out.println("OverlayParticipation = " + overlayParticipation + "%\n");

        int overlayAccessRouterNum = (int) (overlayParticipation * accessRouterNum);
        int overlayBackboneRouterNum = (int) (overlayParticipation * backboneRouterNum);

        System.out.println("OverlayAccessRouters = " + overlayAccessRouterNum + " , AccessRouters = " + (accessRouterNum - overlayAccessRouterNum));
        System.out.println("OverlayBackboneRouters = " + overlayBackboneRouterNum + " , BackboneRouters = " + (backboneRouterNum - overlayBackboneRouterNum) + "\n");
        System.out.println("Random:");
        System.out.println("OverlayAccessRouters = " + randomAccessOverlay + " , AccessRouters = " + (accessRouterNum - randomAccessOverlay));
        System.out.println("OverlayBackboneRouters = " + randomBackboneOverlay + " , BackboneRouters = " + (backboneRouterNum - randomBackboneOverlay) + "\n");

        //Giorgos Nomikos 18/04/2009
        bw.write("\n\n// Konstantinos Katsaros and Giorgos Nomikos 18/04/2009\n");
        bw.write("//	 For Omnet++ version 4:\n");
        bw.write("//	 The following lines must be inserted in the omnetpp.ini (scenario) file for a GT-ITM TS:\n");
	bw.write("//	 (Alternatively the produced GTITM_TS_InetUnderlay-Config.ini must be included by the scenario)\n");
        bw.write("//	 [Config CrescendoInetTopology]\n");
        bw.write("//	 description = Crescendo (semi-recursive, GT-ITM TS IPv4UnderlayNetwork)\n");
        bw.write("//	 network = oversim.underlay.inetunderlay.GTITM_TS_IPv4Network\n");
        bw.write("//	 **.overlayType = \"oversim.overlay.crescendo.CrescendoModules\"\n");
        bw.write("//	 **.tier1Type = \"oversim.applications.kbrtestapp.KBRTestAppModules\"\n");
        bw.write("//	 **.outRouterNum=0\n");
        bw.write("//	 **.startIP = \"10.1.0.1\"\n");
        //bw.write("//	 **.globalObserver.globalFunctionsType=\"GlobalCoordinator\"\n");
        //bw.write("//	 **.globalObserver.useGlobalFunctions=true\n\n");
        //bw.write("//	 One of the following types must be specified.\n");
        bw.write("//	 **.overlayTerminalType = \"OverlayHost\"\n");
        bw.write("//	 **.routingType = \"semi-recursive\"\n");
        bw.write("//	 **.addAccessRouterToOverlay = false\n");

        bw.write("//\t**.accessRouterNum = " + (accessRouterNum - randomAccessOverlay) + "\n");
        bw.write("//\t**.overlayAccessRouterNum = " + randomAccessOverlay + "\n");
        bw.write("//\t**.backboneRouterNum = " + (backboneRouterNum - randomBackboneOverlay) + "\n");
        bw.write("//\t**.overlayBackboneRouterNum = " + randomBackboneOverlay + "\n");

	bw4.write("**.overlayTerminalType = \"OverlayHost\"\n");
        bw4.write("**.accessRouterNum = " + (accessRouterNum - randomAccessOverlay) + "\n");
        bw4.write("**.overlayAccessRouterNum = " + randomAccessOverlay + "\n");
        bw4.write("**.backboneRouterNum = " + (backboneRouterNum - randomBackboneOverlay) + "\n");
        bw4.write("**.overlayBackboneRouterNum = " + randomBackboneOverlay + "\n");
	bw4.write("**.outRouterNum = 0 \n");
	bw4.write("**.startIP = \"10.1.0.1\" \n");
        bw4.write("\n\n");
      
        //---------------------------------------------------------------------------------------
        bw.write("//\n");
        bw.write("// GT-ITM TS topology exported via BRITE.\n");
        bw.write("//\n");

        bw.write("module GTITM_TS_IPv4Underlay\n{\n");
        bw.write("\tparameters:\n");
        bw.write("\t\tstring overlayType;  // the overlay used in the simulation (for overlayBackboneRouters)\n");
        bw.write("\t\tstring tier1Type;    // the application on top of the overlay used in the simulation (for overlayBackboneRouters)\n");
        bw.write("\t\tstring tier2Type;    // the module type on top of tier 1 (for overlayBackboneRouters)\n");
        bw.write("\t\tstring tier3Type;    // the module type on top of tier 2 (for overlayBackboneRouters)\n");
        bw.write("\t\tint accessRouterNum;	// number of access routers in the network\n");
        bw.write("\t\tint overlayAccessRouterNum;	// number of access routers in the network that participate the overlay\n");
        bw.write("\t\tint backboneRouterNum;	// number of backbone routers in the networkor\n");
        bw.write("\t\tint overlayBackboneRouterNum;	// number of backbone routers in the network that participate the overlay\n");
        bw.write("\t\tint outRouterNum;\n");
        bw.write("\t\tstring overlayTerminalType;\n");
//        bw.write("\t\tdouble ttDelay;\n");
//        bw.write("\t\tdouble ttError;\n");
//        bw.write("\t\tdouble ttDatarate;\n");
//        bw.write("\t\tdouble tsDelay;\n");
//        bw.write("\t\tdouble tsError;\n");
//        bw.write("\t\tdouble tsDatarate;\n");
//        bw.write("\t\tdouble ssDelay;\n");
//        bw.write("\t\tdouble ssError;\n");
//        bw.write("\t\tdouble ssDatarate;\n");
        bw.write("\tsubmodules:\n");
        bw.write("\t\tunderlayConfigurator: InetUnderlayConfigurator {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\t@display(\"i=block/cogwheel\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\tglobalObserver: GlobalObserver {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\t@display(\"i=block/control\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\tbackboneRouter[backboneRouterNum]: Router {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\t@display(\"i=device/server2\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\toutRouter[min(1, outRouterNum)]: TunOutRouter {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\t@display(\"i=misc/globe\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\taccessRouter[accessRouterNum]: AccessRouter {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\t@display(\"i=device/server2,gray,30\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\toverlayAccessRouter[overlayAccessRouterNum]: OverlayAccessRouter {\n");
        bw.write("\t\t\tparameters:\n");
        bw.write("\t\t\t\toverlayType = overlayType;    // the overlay used in the simulation\n");
        bw.write("\t\t\t\ttier1Type = tier1Type;    // the application on top off the overly used in the simulation\n");
        bw.write("\t\t\t\ttier2Type = tier2Type;    // tier 2 module to use\n");
        bw.write("\t\t\t\ttier3Type = tier3Type;    // tier 3 module to use\n");
        bw.write("\t\t\t\t@display(\"i=device/server2,gray,30;i2=block/circle_s\");\n");
        bw.write("\t\t}\n");
        bw.write("\t\toverlayBackboneRouter[overlayBackboneRouterNum]: OverlayRouter {\n");
        bw.write("\t\t\t\tparameters:\n");
        bw.write("\t\t\t\toverlayType = overlayType;    // the overlay used in the simulation\n");
        bw.write("\t\t\t\ttier1Type = tier1Type;    // the application on top off the overly used in the simulation\n");
        bw.write("\t\t\t\ttier2Type = tier2Type;    // tier 2 module to use\n");
        bw.write("\t\t\t\ttier3Type = tier3Type;    // tier 3 module to use\n");
        bw.write("\t\t\t\t@display(\"i=device/server2;i2=block/circle_s\");\n");
        bw.write("\t\t}\n");

	//writeASAccessRouterNumInfo(ASRouterNumbers);

        // Konstantinos Katsaros: 21/05/2008
        // We store a map between the ID of a node and its index in the produced array (in order of appearence in the node[])
        for (int i = 0; i < nodes.length; i++)
        {
            Node n = nodes[i];
            NodeConf nc = nodes[i].getNodeConf();
        }

        bw.write("  connections allowunconnected:\n");
        for (int i = 0; i < edges.length; ++i)
        {
            Edge e = edges[i];
            RouterEdgeConf rec = (RouterEdgeConf) e.getEdgeConf();

            int srcID = e.getSrc().getID();
            int destID = e.getDst().getID();

            int srcGateIndex = in_lastused[srcID]++;
            int dstGateIndex = out_lastused[destID]++;

            // Konstantinos Katsaros: 21/05/2008

            Integer srcIDindex = -1;
            Integer dstIDindex = -1;
            String srcArray = "";
            String dstArray = "";

            //Find if the src/dst routers are Access or Backbone routers: 1- AccessRouter, 0- BackboneRouter
            int srcAccess = ((Integer) id2indexAccessArray.get(new Integer(srcID))).intValue();
            int destAccess = ((Integer) id2indexAccessArray.get(new Integer(destID))).intValue();

            if (srcAccess == 1)
            {
                srcIDindex = ((Integer) id2indexAccess.get(new Integer(srcID))).intValue();
            }
            else
            {
                srcIDindex = ((Integer) id2indexBackbone.get(new Integer(srcID))).intValue();
            }

            if (destAccess == 1)
            {
                dstIDindex = ((Integer) id2indexAccess.get(new Integer(destID))).intValue();
            }
            else
            {
                dstIDindex = ((Integer) id2indexBackbone.get(new Integer(destID))).intValue();
            }

            // Find if these routers belong to the overlay: 1- in the overlay array, 0 - otherwise
            Integer srcOverlay = (Integer) id2indexOverlay.get(new Integer(srcID));
            Integer dstOverlay = (Integer) id2indexOverlay.get(new Integer(destID));

            if (srcOverlay == 1)
            {
                if (srcAccess == 1)
                {
                    srcArray = "overlayAccessRouter";
                }
                else
                {
                    srcArray = "overlayBackboneRouter";
                }
            }
            else
            {
                if (srcAccess == 1)
                {
                    srcArray = "accessRouter";
                }
                else
                {
                    srcArray = "backboneRouter";
                }
            }

            if (dstOverlay == 1)
            {
                if (destAccess == 1)
                {
                    dstArray = "overlayAccessRouter";
                }
                else
                {
                    dstArray = "overlayBackboneRouter";
                }
            }
            else
            {
                if (destAccess == 1)
                {
                    dstArray = "accessRouter";
                }
                else
                {
                    dstArray = "backboneRouter";
                }
            }

            System.out.println("[EXPORT]: Link[" + srcArray + ":" + srcIDindex + "," + dstArray + ":" + dstIDindex + "]: ( weight = " + rec.getWeight() + ")");

            // GIORGOS NOMIKOS 02/06/2009
            // For omnet++ 4 we have extended the base Datarate Channel and created a new channel called GTITMChannel
            // which contains parameter for weight.

            // Transit-transit link           
            if ((srcArray.equals("router") && dstArray.equals("router")) || (srcArray.equals("overlayRouter") && dstArray.equals("overlayRouter")) || (srcArray.equals("overlayRouter") && dstArray.equals("router")) || (srcArray.equals("router") && dstArray.equals("overlayRouter")) || (srcArray.equals("backboneRouter") && dstArray.equals("backboneRouter")))
            {
                bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$o++ --> GTITMChannel { ");
                bw.write(" delay = " + /*ttDelay*/rec.getLength() + "ms; ber = " + ttError + "; datarate = " + ttDatarate + "bps; weight = " + rec.getWeight() + ";"); 
                bw.write(" } --> " + dstArray + "[" + dstIDindex.intValue() + "].pppg$i++;\n");

                if (e.getDirection() != GraphConstants.DIRECTED)
                {
                    // duplex
                    srcGateIndex = out_lastused[e.getSrc().getID()]++;
                    dstGateIndex = in_lastused[e.getDst().getID()]++;

                    bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$i++ <-- GTITMChannel { ");
                    bw.write(" delay = " + /*ttDelay*/rec.getLength() + "ms; ber = " + ttError + "; datarate = " + ttDatarate + "bps; weight = " + rec.getWeight() + ";");
                    bw.write(" } <-- " + dstArray + "[" + dstIDindex.intValue() + "].pppg$o++;\n");
                }
            }
            // Stub-stub link            
            else if ((srcArray.equals("accessRouter") && dstArray.equals("accessRouter")) || (srcArray.equals("overlayAccessRouter") && dstArray.equals("overlayAccessRouter")) || (srcArray.equals("overlayAccessRouter") && dstArray.equals("accessRouter")) || (srcArray.equals("accessRouter") && dstArray.equals("overlayAccessRouter")))
            {
                bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$o++ --> GTITMChannel { ");
                bw.write(" delay = " + /*ssDelay*/rec.getLength() + "ms; ber = " + ssError + "; datarate = " + ssDatarate + "bps; weight = " + rec.getWeight() + ";");
                bw.write(" } --> " + dstArray + "[" + dstIDindex.intValue() + "].pppg$i++;\n");

                if (e.getDirection() != GraphConstants.DIRECTED)
                {
                    // duplex
                    srcGateIndex = out_lastused[e.getSrc().getID()]++;
                    dstGateIndex = in_lastused[e.getDst().getID()]++;

                    bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$i++ <-- GTITMChannel { ");
                    bw.write(" delay = " + /*ssDelay*/rec.getLength() + "ms; ber = " + ssError + "; datarate = " + ssDatarate + "bps; weight = " + rec.getWeight() + ";");
                    bw.write(" } <-- " + dstArray + "[" + dstIDindex.intValue() + "].pppg$o++;\n");
                }
            }
            // Transit-stub link            
            else if ((srcArray.equals("router") && dstArray.equals("accessRouter")) || (srcArray.equals("accessRouter") && dstArray.equals("router")) || (srcArray.equals("overlayRouter") && dstArray.equals("overlayAccessRouter")) || (srcArray.equals("overlayAccessRouter") && dstArray.equals("overlayRouter")) || (srcArray.equals("overlayAccessRouter") && dstArray.equals("router")) || (srcArray.equals("router") && dstArray.equals("overlayAccessRouter")) || (srcArray.equals("accessRouter") && dstArray.equals("overlayRouter")) || (srcArray.equals("overlayRouter") && dstArray.equals("accessRouter")) || (srcArray.equals("backboneRouter") && dstArray.equals("accessRouter")) || (srcArray.equals("accessRouterbackboneRouter") && dstArray.equals("backboneRouter")) )
            {
                bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$o++ --> GTITMChannel { ");
                bw.write(" delay = " + /*tsDelay*/rec.getLength() + "ms; ber = " + tsError + "; datarate = " + tsDatarate + "bps; weight = " + rec.getWeight() + ";");
                bw.write(" } --> " + dstArray + "[" + dstIDindex.intValue() + "].pppg$i++;\n");
                if (e.getDirection() != GraphConstants.DIRECTED)
                {
                    // duplex
                    srcGateIndex = out_lastused[e.getSrc().getID()]++;
                    dstGateIndex = in_lastused[e.getDst().getID()]++;

                    bw.write("    " + srcArray + "[" + srcIDindex.intValue() + "].pppg$i++ <-- GTITMChannel { ");
                    bw.write(" delay = " + /*tsDelay*/rec.getLength() + "ms; ber = " + tsError + "; datarate = " + tsDatarate + "bps; weight = " + rec.getWeight() + ";");
                    bw.write(" } <-- " + dstArray + "[" + dstIDindex.intValue() + "].pppg$o++;\n");
                }
            }
            else
            {
                bw.write("ERROR: during topology generation. Please check topology generation script.\n");
            }

        }

        bw.write("}\n");
        bw.write("\n");

        bw.write("//\n");
        bw.write("// Example network that instantiates GTITM_TransitStub_Network.\n");
        bw.write("//\n");
        bw.write("network GTITM_TS_IPv4Network extends GTITM_TS_IPv4Underlay\n{\n");
        bw.write("\tparameters:\n}\n");
        bw.write("\n");

        bw.close();
        bw2.close();
        //bw3.close();
	bw4.close();
        Util.MSG("... DONE.");
    }


    public static void convert(String briteFile, int format) throws Exception
    {
        FileModel f = new FileModel(Import.ImportConstants.BRITE_FORMAT, briteFile, format);
        Topology t = new Topology(f);
        Omnetpp4Export oe = new Omnetpp4Export(t, new File(briteFile + "_omnetpp.ned"));
        System.out.println("In convert...");
        oe.export();
    }

    public static void main(String args[]) throws Exception
    {
        String briteFile = "";
        String routeroras = "";
        try
        {
            briteFile = args[0];
            routeroras = args[1];
        }
        catch (Exception e)
        {
            Util.ERR("Usage:  java Export.Omnetpp4Export <brite-format-file> RT {| AS}");
        }

        int format = ModelConstants.RT_FILE;
        if (routeroras.equalsIgnoreCase("as"))
        {
            format = ModelConstants.AS_FILE;
        }

        FileModel f = new FileModel(Import.ImportConstants.BRITE_FORMAT, briteFile, format);

        Topology t = new Topology(f);
        Omnetpp4Export oe = new Omnetpp4Export(t, new File(briteFile + "_omnetpp.ned"));
        System.out.println("In main...");
        oe.export();
    }
}

