package Export;

import Topology.*;
import Graph.*;
import Model.*;
import Util.*;

import java.io.*;
import java.util.*;


/**
 * Export filter for OMNeT++
 */
public class OmnetppExport {

    private Topology t;
    private BufferedWriter bw;
    private BufferedWriter bw2;
    private BufferedWriter bw3;
    private BufferedReader br;
    private String channelsFullFileName;
    private String channelsFileName;

    //NTINOS KATSAROS: 21/05/2008
    // HashMaps that will help as keep the mapping between BRITE node IDs and their index in the created
    // router sub-module arrays. The id2indexOverlay will help as keep track of overlay participation.
    private HashMap id2indexAccess;
    private HashMap id2indexBackbone;
    private HashMap id2indexOverlay; //id2indexOverlay.second = 1 => in overlay
    private HashMap id2indexAccessArray; //.second = 1 => is an access router


    public OmnetppExport(Topology t, String outFile) {
        this.t = t;
        try {
            bw = new BufferedWriter(new FileWriter(new File(outFile+".ned")));
	    
	    //NTINOS KATSAROS: 25/06/2008
	    bw2 = new BufferedWriter(new FileWriter(new File (outFile+"-ASIDs.ini")));
		bw2.write("[Parameters]\n");

	    //NTINOS KATSAROS: 03/04/2009
		//Will write the number of access routers per stub domain.
	    bw3 = new BufferedWriter(new FileWriter(new File (outFile+"-AS_AccessRouterNum.ini")));
		bw3.write("[Parameters]\n");
		
	    //NTINOS KATSAROS: 25/06/2008
	    //No need for a new ned file with channels definitions. Will insert these attributes
	    // directly in the connections section, avoiding this way the overhead of connection-channel 
	    // mapping information maintenance.
        }
        catch (IOException e) {
            Util.ERR("Error creating BufferedWriter in OmnetppExport: " +e);
        }
    }

/*
    public void channelsFullFileName(File file)
    {
	String fullfilename = file.getPath();
	String filename = file.getName();
	channelsFileName = filename.substring(0,filename.indexOf('.'))+"_Channels";	
	channelsFullFileName = fullfilename.substring(0,fullfilename.indexOf('.'))+"_Channels";
    }
*/

public void writeASIDinfo(int accessRouterIndex, int ASID)
{ 
	try 
	{
		bw2.write("**.accessRouter["+accessRouterIndex+"].ASID = "+ASID+"\n");
	}
    catch (Exception e) {
            Util.ERR("Error writting ASID parameter values: " +e);
        }
}

public void writeASAccessRouterNumInfo(HashMap map)
{ 
	try 
	{
		//bw3.write("**.accessRouter["+accessRouterIndex+"].ASID = "+ASID+"\n");
		bw.write("    ASInfoArray: ASInfo["+map.size()+"]\n");
		bw.write("      display: \"i=block/control\";\n");	

		int i=0;
		for (Iterator it = map.entrySet().iterator(); it.hasNext();) 
		{
			Map.Entry entry = (Map.Entry)it.next();
			Integer ASID = (Integer)entry.getKey();
			Integer RouterNumber = (Integer)entry.getValue() ;
			bw3.write("**.ASInfoArray["+i+"].ASID = "+ASID+"\n");
			bw3.write("**.ASInfoArray["+i+"].ASaccessRouterNum = "+RouterNumber+"\n");
			i++;
		}
	}
    catch (Exception e) {
            Util.ERR("Error writting ASID parameter values: " +e);
        }
}


    public void export() throws Exception {
        Util.MSG("Producing export file for OMNeT++ ");
        Graph g = t.getGraph();
	FileModel model = (FileModel)t.getModel();
	double overlayParticipation = model.getOverlayParticipation();
	System.out.println("[EXPORT]: OverlayParticipation: " +overlayParticipation);
	//NTINOS KATSAROS: 21/05/2008
	id2indexAccess = new HashMap();
	id2indexBackbone = new HashMap();
	id2indexOverlay = new HashMap();
	id2indexAccessArray = new HashMap();
	HashMap ASRouterNumbers= new HashMap();

        Node[] nodes = g.getNodesArray();

        Arrays.sort(nodes, Node.IDcomparator);
        Edge[] edges = g.getEdgesArray();
        Arrays.sort(edges, Edge.SrcIDComparator);

        int maxid=0;
        for (int i=0; i<nodes.length; i++)
        {
            int id = nodes[i].getID();
            if (id>maxid)
                maxid = id;
        }
        int in_lastused[] = new int[maxid+1];
        int out_lastused[] = new int[maxid+1];
        for (int i=0; i<maxid; i++)
        {
            in_lastused[i] = 0;
            out_lastused[i] = 0;
        }

        bw.write("//\n");
        bw.write("// OMNeT++ network description -- exported from GT-ITM (alt) topology\n");
        bw.write("// Generator Model Used:\n");
        StringTokenizer st = new StringTokenizer(t.getModel().toString(), "\n");
        while (st.hasMoreTokens())
        {
            bw.write("//   "+st.nextToken()+"\n");
        }	
        bw.write("//\n");
        bw.write("\n\n");

        bw.write("//\n");
        bw.write("// Prototype for modules that can be used as node in a GTITM_TransitStub_Network\n");
        bw.write("//\n");

        bw.write("import\n");
        bw.write("    \"Tiers\",\n");
        bw.write("    \"Overlay\",\n");
        bw.write("    \"Router\",\n");
        bw.write("    \"AccessRouter\",\n");
        bw.write("    \"OverlayRouter\",\n");
        bw.write("    \"OverlayAccessRouter\",\n");
        bw.write("    \"TunOutRouter\",\n");
        bw.write("    \"IPv4UnderlayConfigurator\",\n");
        bw.write("    \"StandardHost\",\n");
        bw.write("    \"GlobalObserver\";\n\n");
        //bw.write("    \""+channelsFileName+"\"; //NTINOS KATSAROS 25/06/2008: Have to include the channels deescription file.\n\n");

	//NTINOS KATSAROS: 15/05/2008
	// We do this loop in order tou count the access routers. We need this info as it is retrieved by the IPv4UnderlayConfiguratorModule
	//NTINOS KATSAROS: 21/05/2008
	//In this first parsing we also keep the mapping between the BRITE node IDs and the corresponding sub-module arrays 
	// (that will be produced later) indexes.
	int accessRouterNum = 0;
	int backboneRouterNum = 0;
	int accessRouterIndex = -1;
	int backboneRouterIndex = -1;
	int randomAccessOverlay = 0;
	int randomBackboneOverlay = 0;
	for (int i=0; i<nodes.length; i++)
	{
	    Node n = nodes[i];
	    NodeConf nc = nodes[i].getNodeConf();
	    RouterNodeConf rnc = (RouterNodeConf) nc;
	
	   if (rnc.getType()==ModelConstants.RT_STUB)
	   {
			id2indexAccessArray.put(new Integer(n.getID()),1);
			accessRouterNum++;
			if (Math.random()<(overlayParticipation)) 
			{
				randomAccessOverlay++;
				//Storing in which array this node will be
				id2indexOverlay.put(new Integer(n.getID()),1);
				//Storing its index there
				id2indexAccess.put(new Integer(n.getID()),new Integer(randomAccessOverlay-1));
			}
			else
			{ 
				id2indexOverlay.put(new Integer(n.getID()),0);
				accessRouterIndex++;
				id2indexAccess.put(new Integer(n.getID()),new Integer(accessRouterIndex));
				
				writeASIDinfo(accessRouterIndex, rnc.getASID());
				Integer ASIDobj = rnc.getASID();
				Integer storedEntry = (Integer)ASRouterNumbers.get(ASIDobj);
				if (storedEntry == null)
					 ASRouterNumbers.put(new Integer(rnc.getASID()), new Integer(1));
				else
				{
					ASRouterNumbers.remove(ASIDobj);
					ASRouterNumbers.put(ASIDobj,new Integer(storedEntry+1));
				}
			}
		
		
		
	   }
	   else if (rnc.getType()==ModelConstants.RT_BACKBONE)
	   {
		id2indexAccessArray.put(new Integer(n.getID()),0);
		backboneRouterNum++;
		if (Math.random()<(overlayParticipation)) 
		{
			randomBackboneOverlay++;
			id2indexOverlay.put(new Integer(n.getID()),1);
			id2indexBackbone.put(new Integer(n.getID()),new Integer(randomBackboneOverlay-1));
		}
		else 
		{
			id2indexOverlay.put(new Integer(n.getID()),0);
			backboneRouterIndex++;
			id2indexBackbone.put(new Integer(n.getID()),new Integer(backboneRouterIndex));
		}
		
		
	   }
	}

	

	System.out.println("We have totaly:\n");
	System.out.println("AccessRouters = "+accessRouterNum);
	System.out.println("BackboneRouters = "+backboneRouterNum);
	System.out.println("OverlayParticipation = "+overlayParticipation+"%\n");

	int overlayAccessRouterNum = (int)(overlayParticipation*accessRouterNum);
	int overlayBackboneRouterNum = (int)(overlayParticipation*backboneRouterNum);

	System.out.println("OverlayAccessRouters = "+overlayAccessRouterNum+" , AccessRouters = "+(accessRouterNum - overlayAccessRouterNum));
	System.out.println("OverlayBackboneRouters = "+overlayBackboneRouterNum+" , BackboneRouters = "+(backboneRouterNum - overlayBackboneRouterNum)+"\n");
	System.out.println("Random:");
System.out.println("OverlayAccessRouters = "+randomAccessOverlay+" , AccessRouters = "+(accessRouterNum - randomAccessOverlay));
	System.out.println("OverlayBackboneRouters = "+randomBackboneOverlay+" , BackboneRouters = "+(backboneRouterNum - randomBackboneOverlay)+"\n");
	
	bw.write("//	 Ntinos Katsaros: 22/05/2008\n");
	bw.write("//	 The following lines must be inserted in the omnetpp.ini (scenario) file for a GT-ITM TS:\n\n");
	bw.write("//	 [Run ##]\n");
	bw.write("//	 description = \"GTITM TS Scribe Test\"\n");
	bw.write("//	 network = GTITM_TS_IPv4Network\n");

	bw.write("//	 **.overlay.measureNetwInitPhase=true                #variable: gather statistics when bootstrapping\n");
	bw.write("//	 **.overlayType = \"PastryModules\"\n");
	bw.write("//	 **.tier1Type = \"MulticastScribe\"\n");
	bw.write("//	 **.tier2Type = \"ScribeTestModule\"\n");
	bw.write("//	 **.routerTier1Type = \"MulticastScribe\"\n");
	bw.write("//	 **.routerTier2Type = \"ScribeTestModule\"\n");
	bw.write("//	 **.numTiers=2\n");
	bw.write("//	 **.overlay.iterativeLookup=false\n");
	bw.write("//	 **.overlay.useCommonAPIforward=true\n");
	bw.write("//	 **.overlay.pastry.optimizeLookup=false              #whether to search the closest node in findCloserNode() calls\n");
	bw.write("//	 **.overlay.pastry.optimisticForward=false           #forward message immediately in recursive mode, otherwise ping first\n");
	bw.write("//	 **.overlay.pastry.avoidDuplicates=false             # when node seems unreachable but msg already sent, do not retry\n");
	bw.write("//	 **.outRouterNum=0\n");
	bw.write("//	 **.startIP = \"10.1.0.1\"\n");
	bw.write("//	 **.globalObserver.globalFunctionsType=\"GlobalCoordinator\"\n");
	bw.write("//	 **.globalObserver.useGlobalFunctions=1\n\n");
	bw.write("//	 One of the following types must be specified.\n");
	bw.write("//	 **.overlayTerminalType = \"OverlayHost\"\n");
	bw.write("//	 **.overlayTerminalType = \"StandardHost\"\n");
	bw.write("//	 **.overlayTerminalType = \"BTHost\"\n");
	bw.write("//	 **.overlayTerminalType = \"MulticastBTHost\"\n\n");
	bw.write("//	 These are the link attribute values. We may give different values for different types of links\n");
	bw.write("//	 i.e. transit-transit, transit-stub, stub-stub.\n\n");		

	bw.write("//	 **.ttDelay = 0.0018,\n");
	bw.write("//	 **.ttError = 1e-6,\n");
	bw.write("//	 **.ttDatarate = 128000,\n");
	bw.write("//	 **.tsDelay = 0.0017,\n");
	bw.write("//	 **.tsError = 1e-7,\n");
	bw.write("//	 **.tsDatarate = 17000,\n");
	bw.write("//	 **.ssDelay = 0.0016,\n");
	bw.write("//	 **.ssError = 1e-8,\n");
	bw.write("//	 **.ssDatarate = 126000,\n");

	bw.write("//       **.addAccessRouterToOverlay = true,\n");

	bw.write("//\t**.accessRouterNum = "+(accessRouterNum - randomAccessOverlay)+"\n");
	bw.write("//\t**.overlayAccessRouterNum = "+randomAccessOverlay+"\n");
	bw.write("//\t**.backboneRouterNum = "+(backboneRouterNum - randomBackboneOverlay)+"\n");
	bw.write("//\t**.overlayBackboneRouterNum = "+randomBackboneOverlay+"\n");
	bw.write("//\n\n");
	//---------------------------------------------------------------------------------------
    bw.write("//\n");
    bw.write("// GT-ITM TS topology exported via BRITE.\n");
    bw.write("//\n");

    bw.write("module GTITM_TS_IPv4Underlay\n");
    bw.write("  parameters:\n");

    //NTINOS KATSAROS: 15/05/2008
	//Parameters derived from IPv4Underlay
    bw.write("      overlayType: string,  // the overlay used in the simulation (for overlayBackboneRouters)\n");
    bw.write("      routerTier1Type: string,    // the application on top of the overlay used in the simulation (for overlayBackboneRouters)\n");
    bw.write("      routerTier2Type: string,    // the module type on top of tier 1 (for overlayBackboneRouters)\n");
	bw.write("      routerTier3Type: string,    // the module type on top of tier 2 (for overlayBackboneRouters)\n");
	bw.write("      accessRouterNum: numeric,	// number of access routers in the network\n");   
    bw.write("      overlayAccessRouterNum: numeric,	// number of access routers in the network that participate the overlay\n");    
 	bw.write("      backboneRouterNum: numeric,	// number of backbone routers in the networkor\n");    
    bw.write("      overlayBackboneRouterNum: numeric,	// number of backbone routers in the network that participate the overlay\n");    
	bw.write("      outRouterNum: numeric,\n");
	//bw.write("      overlayRouters: bool,\n");
	bw.write("      overlayTerminalType: string,\n");
	bw.write("      numProxyRouters: numeric const, //number of Routers providing overlay multicast support. If negative then stands for a percentage (%)\n");
	bw.write("      numProxyASs: numeric const, //number of ASs providing router support for overlay multicast. If negative then stands for a percentage (%)\n");
	//bw.write("      trackerTCPAppType: string,\n");
    //bw.write("      addAccessRouterToOverlay: bool,\n");
	bw.write("      ttDelay: numeric,\n");
	bw.write("      ttError: numeric,\n");
	bw.write("      ttDatarate: numeric,\n");
	bw.write("      tsDelay: numeric,\n");
	bw.write("      tsError: numeric,\n");
	bw.write("      tsDatarate: numeric,\n");
	bw.write("      ssDelay: numeric,\n");
	bw.write("      ssError: numeric,\n");
	bw.write("      ssDatarate: numeric;\n");


	//NTINOS KATSAROS: 15/05/2008
	//This submodule will be used to set up our network (also derived from IPv4Underlay).
    bw.write("  submodules:\n");
	bw.write("    underlayConfigurator: IPv4UnderlayConfigurator;\n");
	bw.write("      display: \"i=block/cogwheel\";\n");
    bw.write("    globalObserver: GlobalObserver;\n");
    bw.write("      display: \"i=block/control\";\n");
	bw.write("    backboneRouter: Router[backboneRouterNum];\n");
	bw.write("      display: \"i=device/server2\";\n");
	bw.write("    outRouter: TunOutRouter[min(1, outRouterNum)];\n");
	bw.write("      display: \"i=misc/globe\";\n");
	bw.write("    accessRouter: AccessRouter[accessRouterNum];\n");
	bw.write("      display: \"i=device/server2,gray,30\";\n");
	bw.write("    overlayAccessRouter: OverlayAccessRouter[overlayAccessRouterNum]\n");
	bw.write("      parameters:\n");
	bw.write("        overlayType = overlayType,    // the overlay used in the simulation\n");
	bw.write("        tier1Type = routerTier1Type,    // the application on top off the overly used in the simulation\n");
	bw.write("        tier2Type = routerTier2Type,    // tier 2 module to use\n");
	bw.write("        tier3Type = routerTier3Type;    // tier 3 module to use\n");
	bw.write("      display: \"i=device/server2,gray,30;i2=block/circle_s\";\n");
	bw.write("    overlayBackboneRouter: OverlayRouter[overlayBackboneRouterNum]\n");
	bw.write("      parameters:\n");
	bw.write("        overlayType = overlayType,    // the overlay used in the simulation\n");
	bw.write("        tier1Type = routerTier1Type,    // the application on top off the overly used in the simulation\n");
	bw.write("        tier2Type = routerTier2Type,    // tier 2 module to use\n");
	bw.write("        tier3Type = routerTier3Type;    // tier 3 module to use\n");
	bw.write("      display: \"i=device/server2;i2=block/circle_s\";\n");	
	writeASAccessRouterNumInfo(ASRouterNumbers);
	bw.write("//    tracker: StandardHost;\n");
	bw.write("//      display: \"i=device/server_l\";\n");
	//NTINOS KATSAROS: 21/05/2008
	// We store a map between the ID of a node and its index in the produced array (in order of appearence in the node[])
	for (int i=0; i<nodes.length; i++)
    {
    	Node n = nodes[i];
        NodeConf nc = nodes[i].getNodeConf();
    }

    bw.write("  connections nocheck:\n");
    for (int i=0; i<edges.length; ++i)
    {
	    Edge e = edges[i];
	    RouterEdgeConf rec = (RouterEdgeConf)e.getEdgeConf();

		int srcID = e.getSrc().getID();
        int destID = e.getDst().getID();

        int srcGateIndex = in_lastused[srcID]++;
        int dstGateIndex = out_lastused[destID]++;

	    //NTINOS KATSAROS: 21/05/2008

		Integer srcIDindex = -1;
		Integer dstIDindex = -1;
		String srcArray = "";
		String dstArray = "";

		//Find if the src/dst routers are Access or Backbone routers: 1- AccessRouter, 0- BackboneRouter
		int srcAccess = ((Integer) id2indexAccessArray.get(new Integer(srcID))).intValue();
		int destAccess = ((Integer) id2indexAccessArray.get(new Integer(destID))).intValue();
		
		if (srcAccess==1)
			srcIDindex = ((Integer) id2indexAccess.get(new Integer(srcID))).intValue();
		else
			srcIDindex = ((Integer) id2indexBackbone.get(new Integer(srcID))).intValue();
		
		if (destAccess==1)
			dstIDindex = ((Integer) id2indexAccess.get(new Integer(destID))).intValue();
		else 
			dstIDindex = ((Integer) id2indexBackbone.get(new Integer(destID))).intValue();
		
		// Find if these routers belong to the overlay: 1- in the overlay array, 0 - otherwise
		Integer srcOverlay = (Integer) id2indexOverlay.get(new Integer(srcID));
		Integer dstOverlay = (Integer) id2indexOverlay.get(new Integer(destID));
		
		if (srcOverlay==1) 
		{
			if (srcAccess==1) srcArray = "overlayAccessRouter";
			else srcArray = "overlayBackboneRouter";
		}
		else 
		{
			if (srcAccess==1) srcArray = "accessRouter";
			else srcArray = "backboneRouter";
		}
		
		if (dstOverlay==1) 
		{
			if (destAccess==1) dstArray = "overlayAccessRouter";
			else dstArray = "overlayBackboneRouter";
		}
		else 
		{
			if (destAccess==1) dstArray = "accessRouter";
			else dstArray = "backboneRouter";
		}
	
	//System.out.println("[EXPORT]: Link["+srcArray+":"+srcIDindex+","+dstArray+":"+dstIDindex+"]: (length = "+rec.getLength()+ " , weight = "+rec.getWeight()+")");

	    //NTINOS KATSAROS 25/06/2008
	    // Since the channel definition structure has a fixed number of attributed (i.e. delay, error, datarare), we use
	    // some of them to store the desired information (i.e. length, weight). Later, we will dynamically
	    // add new attributes for this information and fix the delay and error values. This is necessary because we have 
	    // to keep this information in a ned file due to the offline nature of the topology generation process. The correct, 
 	    // but time-consuming way to do this would be to extend the channel definition AND the NED parser.

            bw.write("    "+srcArray+"["+srcIDindex.intValue()+"].out++ -->");
	    bw.write(" delay "+rec.getLength()+" error "+rec.getWeight()+" datarate "+e.getBW()*1000000);
            bw.write("  --> "+dstArray+"["+dstIDindex.intValue()+"].in++;\n");

            if (e.getDirection() != GraphConstants.DIRECTED)
            {
                // duplex
                srcGateIndex = out_lastused[e.getSrc().getID()]++;
                dstGateIndex = in_lastused[e.getDst().getID()]++;
  	        bw.write("    "+srcArray+"["+srcIDindex.intValue()+"].in++ <--");
		bw.write(" delay "+rec.getLength()+" error "+rec.getWeight()+" datarate "+e.getBW()*1000000);
		bw.write(" <-- "+dstArray+"["+dstIDindex.intValue()+"].out++;\n");

            }
        }

        bw.write("endmodule\n");
        bw.write("\n");

        bw.write("//\n");
        bw.write("// Example network that instantiates GTITM_TransitStub_Network.\n");
        bw.write("//\n");
	bw.write("network GTITM_TS_IPv4Network:  GTITM_TS_IPv4Underlay\n");
        bw.write("endnetwork\n");
        bw.write("\n");

        bw.close();
		bw2.close();
		bw3.close();
        Util.MSG("... DONE.");
    }


    public static void convert(String briteFile, int format) throws Exception {
        FileModel f = new FileModel(Import.ImportConstants.BRITE_FORMAT, briteFile, format);
        Topology t = new Topology(f);
        OmnetppExport oe = new OmnetppExport(t, briteFile+"_omnetpp.ned");
        System.out.println("In convert...");
        oe.export();
    }


    public static void main(String args[]) throws Exception {
        String briteFile = "";
        String routeroras = "";
        try {
            briteFile = args[0];
            routeroras = args[1];
        }
        catch (Exception e) {
            Util.ERR("Usage:  java Export.OmnetppExport <brite-format-file> RT {| AS}");
        }

        int format = ModelConstants.RT_FILE;
        if (routeroras.equalsIgnoreCase("as"))
            format = ModelConstants.AS_FILE;

        FileModel f = new FileModel(Import.ImportConstants.BRITE_FORMAT, briteFile, format);

        Topology t = new Topology(f);
		OmnetppExport oe = new OmnetppExport(t,briteFile+"_omnetpp.ned");
        //OmnetppExport oe = new OmnetppExport(t, new File(briteFile+"_omnetpp.ned"));
        System.out.println("In main...");
	oe.export();
    }

}

